@@ -7,14 +7,14 @@ use warnings;
 use Scalar::Util qw/reftype refaddr/;
 use Encode qw/decode encode/;
 
-our $VERSION = '0.06';
+our $VERSION = '0.07';
 our (%HELP, @MAYBE, $caller, $destruct);
 BEGIN {
 	%HELP = (
 		arrayref => sub { return map { $_[0]->($_, $_[2]) } @{ $_[1] } },
 		hashref => sub { $caller->can('filter_keys') && $caller->filter_keys($_[1]->{$_}, $_) and next or 
 			$destruct && do { $_[3]{$_[0]->($_)} = $_[0]->($_[1]->{$_}, $_[2]) } || do { $_[1]->{$_} = $_[0]->($_[1]->{$_}, $_[2]) } for keys %{ $_[1] }; $_[3]; },
-		scalarref => sub { my $s = $_[1]; ${$s} =~ m/^[0-9.]+$/g ? $s : do { $s = $_[0]->(${$s}); $destruct ? $s : \$s; } }, 
+		scalarref => sub { ${$_[1]} =~ m/^[0-9.]+$/g ? $_[1] : do { ${$_[1]} =~ s/^(.*)$/$_[0]->(${$_[1]})/e; $_[1]; } && $destruct ? ${$_[1]} : $_[1]; }, 
 		scalar => sub { eval { $_[1] = $_[0]->($_, $_[1], Encode::FB_CROAK); 1; } and last foreach @MAYBE; $_[1]; }
 	);
 }
@@ -48,16 +48,12 @@ sub _maybe {
 
 sub _recurse {
 	my $addr = refaddr $_[0];
-	return ($_[1] eq 'SCALAR') ? exists $_[3]->{$addr} ? $_[3]->{$addr} : do { $_[3]->{$addr} = $HELP{scalarref}->($_[2], $_[0]) } && $_[3]->{$addr}
-		: exists $_[3]->{$addr} ? $_[0] : do { $_[3]->{$addr} = 1 } && $_[1] eq 'ARRAY' ? $HELP{arrayref}->($_[2], $_[0], $_[3]) && $_[0] : $_[1] eq 'HASH' 
-			? $HELP{hashref}->($_[2], $_[0], $_[3], 1) && $_[0] : $_[0];
+	return exists $_[3]->{$addr} ? $_[3]->{$addr} : do { $_[3]->{$addr} = 1 } && $_[1] eq 'SCALAR' ? $HELP{scalarref}->($_[2], $_[0]) : $_[1] eq 'ARRAY'
+		? $HELP{arrayref}->($_[2], $_[0], $_[3]) && $_[0] : $_[1] eq 'HASH' ? $HELP{hashref}->($_[2], $_[0], $_[3], 1) && $_[0] : $_[0];
 }
 
 sub _d_recurse {
-	return $HELP{scalarref}->($_[2], $_[0]) if ( $_[1] eq 'SCALAR' );
-	return [ $HELP{arrayref}->($_[2], $_[0]) ] if ($_[1] eq 'ARRAY');
-	return $HELP{hashref}->($_[2], $_[0], {}) if ($_[1] eq 'HASH');
-	return $_[0];
+	return $_[1] eq 'SCALAR' ?  $HELP{scalarref}->($_[2], $_[0]) : $_[1] eq 'ARRAY' ? [ $HELP{arrayref}->($_[2], $_[0]) ] : $_[1] eq 'HASH' ? $HELP{hashref}->($_[2], $_[0], {}) : $_[0];
 }
 
 1;
@@ -70,7 +66,7 @@ Struct::WOP - deeply encode/decode a struct
 
 =head1 VERSION
 
-Version 0.06
+Version 0.07
 
 =cut
 
